%%%-------------------------------------------------------------------
%%% @doc    衣橱系统-头文件
%%% Module  : wardrobe.hrl
%%% Created : 2020-11-16
%%% @Author : tyl
%%% @end
%%%-------------------------------------------------------------------

%% 大类
-define(WARDROBE_TYPE_FASHION, 1).       %% 时装
-define(WARDROBE_TYPE_DESG,    2).       %% 称号
-define(WARDROBE_TYPE_PICTURE, 3).       %% 头像
-define(WARDROBE_TYPE_FRAME,   4).       %% 相框
-define(WARDROBE_TYPE_BUBBLE,  5).       %% 气泡
-define(WARDROBE_TYPE_FOSTER_SKIN,   6). %% 珍宝
-define(WARDROBE_TYPE_CHILD_FASHION, 7). %% 宝宝时装
-define(WARDROBE_TYPE_GALAXY,   8).      %% 星辰
-define(WARDROBE_TYPE_FOSTER,   9).      %% 进阶
-define(WARDROBE_TYPE_PARTNER, 10).      %% 伙伴

-define(ALL_WARDROBE_TYPE, [1,2,3,4,5,6,7,8,9,10]). %% 所有类型

%% 各大类下的子类型
%% --------------- 时装专用 --------------------
-define(FASHION_SUB_TYPE_1, 1). %% 服饰
-define(FASHION_SUB_TYPE_2, 2). %% 头饰
-define(FASHION_SUB_TYPE_3, 3). %% 背饰

%% --------------- 珍宝专用 --------------------
-define(FOSTER_SKIN_SUB_TYPE_1, 1). %% 座驾
-define(FOSTER_SKIN_SUB_TYPE_2, 2). %% 炫翼
-define(FOSTER_SKIN_SUB_TYPE_3, 3). %% 宝具
-define(FOSTER_SKIN_SUB_TYPE_4, 4). %% 武器
-define(FOSTER_SKIN_SUB_TYPE_5, 5). %% Ai娘

%% --------------- 进阶专用 --------------------
-define(FOSTER_SUB_TYPE_1, 1). %% 座驾
-define(FOSTER_SUB_TYPE_2, 2). %% 炫翼
-define(FOSTER_SUB_TYPE_3, 3). %% 宝具
-define(FOSTER_SUB_TYPE_4, 4). %% 武器

%% --------------- 伙伴专用 --------------------
-define(PARTNER_SUB_TYPE_1, 1). %% Ai娘
-define(PARTNER_SUB_TYPE_2, 2). %% 磁炮
-define(PARTNER_SUB_TYPE_3, 3). %% 星翼
-define(PARTNER_SUB_TYPE_4, 4). %% 幻甲

%% 商城限购类型
-define(WARDROBE_STORE_NOT_LIMIT,  0).   %% 不限购
-define(WARDROBE_STORE_WEEK_LIMIT, 1).   %% 周限购
-define(WARDROBE_STORE_DAY_LIMIT,  2).   %% 日限购

%% 衣橱
-record(status_wardrobe, {
	wardrobe_value = 0      %% 风采总值
	,stage         = 0      %% 阶数
	,wardrobe_map  = #{}    %% 各类型风采数据#{type=>#wardrobe_appearance{}...}
}).

%% 各类型外观风采数据
-record(wardrobe_appearance, {
	type            = 0     %% 类型
	,wardrobe_value = 0     %% 风采值
	,collect        = 0     %% 当前收集进度
	,collect_sum    = 0     %% 配置总收集进度(每个外观统计配置)
}).

%% ------------------- 配置 -------------------------------------
%% 货柜开启条件
-record(base_wardrobe_open, {
	store_id    = 0     %% 货柜Id
	,store_name = ""    %% 名称
	,condition  = 0     %% 解锁条件
	,icon       = 0     %% 图标展示
}).

%% 积分商城
-record(base_wardrobe_store, {
	store_id    = 0     %% 货柜Id
	,goods_id   = 0     %% 物品Id
	,goods_num  = 0     %% 物品数量
	,money_type = 0     %% 价格类型
	,price      = 0     %% 价格
	,discount   = 0     %% 折扣
	,limit_type = 0     %% 限购类型
	,limit_num  = 0     %% 限购数量
	,tag        = 0     %% 标签
	,career     = {}    %% 职业
	,is_top     = 0     %% 是否置顶(主打)
	,pos        = 0     %% 展示位置
	,condition  = []    %% 购买条件(等级限制)
}).

%% 风采值阶数
-record(base_wardrobe_stage, {
	stage              = 0      %% 阶数
	,low_value         = 0      %% 风采值下限
	,high_value        = 0      %% 风采值上限
	,icon              = 0      %% 图标Id
}).

%% 风采值类型
-record(base_wardrobe_appearance, {
	type                = 0     %% 类型
	,sub_type           = 0     %% 子类型
	,level              = 0     %% 品质
	,appearance_value   = 0     %% 风采值
	,goods_type         = []    %% 对应的物品类型(前端读取)
	,name               = ""    %% 名称
}).

%% ------------------- db操作 -------------------------------------
-define(SQL_INSERT_WARDROBE_VALUE, <<"REPLACE INTO wardrobe_value (`role_id`, `type`, `collect`, `value`) VALUES (~p, ~p, ~p, ~p)">>).
-define(SQL_SELECT_WARDROBE_VALUE, <<"SELECT `type`,`collect`,`value` FROM wardrobe_value WHERE `role_id` = ~p">>).
