%%-----------------------------------------------------------------------------
%%
%% the crontab running in erlang
%%
%% 1. 基本格式(Format):
%% {{m,  h, dom, mon, dow}, {M, F, A}}.
%% {{分, 时, 日, 月, 周}, {M, F, A}}
%% M:F(A)在新建的一个独立临时进程运行
%%
%% 2. 字段取值范围(Field Range):
%%  field               allowed values
%%  -----               --------------
%%  m(minute)           0-59
%%  h(hour)             0-23
%%  dom(day of month)   1-31
%%  mon(month)          1-12
%%  dow(day of week)    1-7
%%
%% 3. 用例(Usage):
%% {{"*/1", "*", "*", "*", "*"}, {io, format, ["crontab~n"]}}.
%% 每分钟输出crontab
%%
%% {{"*", "*/1", "*", "*", "*"}, {io, format, ["crontab~n"]}}.
%% 每小时输出crontab
%%
%% {{0, 3, "*", "*", "*"}, {io, format, ["crontab~n"]}}.
%% 每天凌晨3:0输出crontab
%%
%% {{0, 3, "*", "*", 1}, {io, format, ["crontab~n"]}}.
%% 每周星期一凌晨3:0输出crontab
%%
%% {{0, 21, "1-3,26-28", "*", "*"}, {io, format, ["crontab~n"]}}.
%% 每月1-3号,26-28号的晚上21:00输出crontab
%%
%% {{0, "0-23/2", "*", "*", "*"}, {io, format, ["crontab~n"]}}.
%% 每天0-23小时每隔2小时,即(0,2,4,6,8 ...22)输出crontab
%%-----------------------------------------------------------------------------

%% 游戏服crontab配置
-module(data_crontab).
-export([get_config/0]).

get_config() ->
    [
        %% 每10分钟清理一次过期掉落
        {{"*/10", "*", "*", "*", "*"}, {mod_drop, clean_drop, []}},

        %% 每10分钟执行一次在线人数上报到TA系统
        {{"*/10", "*", "*", "*", "*"}, {ta_agent_fire, online_track, []}},

        %% 每5分钟更新榜单
        % {{"*/5", "*", "*", "*", "*"}, {lib_common_rank_api, timer_common_rank, []}},
        %% {{45, "*", "*", "*", "*"}, {lib_common_rank_api, timer_common_rank, []}},

        %% 邮件推送
        {{"*/1", "*", "*", "*", "*"}, {mod_pushmail, timer_check, []}},

        %% 活动开启
        {{"*/1", "*", "*", "*", "*"}, {mod_activitycalen, timer_check, []}},

        %% 每日01点59分清理过期数据
        {{59, 1, "*", "*", "*"}, {svr_vitality_local_mgr, timer_clean_outdate, []}},

        %% 定制活动开启
        {{"*/1", "*", "*", "*", "*"}, {mod_custom_act, apply_cast, [timer_check, []]}},

        %% 每日0点10分,检查每日累充消费活动结算
        {{5, 0, "*", "*", "*"}, {custom_act_pellet, daily_settle, []}},

        %% *:0分刷新公会战力
        {{0, "*/1", "*", "*", "*"}, {mod_guild, count_guild_combat_power, []}},

        %% 巅峰竞技：每赛季结束次日0:10结算
        {{2, 0, "*", "*", "*"}, {mod_top_pk_local, daily_update, []}},

        %% 拍卖行：每分钟轮询一次检测拍卖活动开始以及拍卖结束
        %{{"*/1", "*", "*", "*", "*"}, {mod_auction, timer_check, []}},

        %% 每小时社团徽章更新
        {{0, "0-23/1", "*", "*", "*"}, {mod_common_rank, timer_update, []}},

        %% 每15分钟计算一次
        {{"*/15", "*", "*", "*", "*"}, {mod_mobilize_mgr, timer_check, []}},

        %% 限时邮件
        {{"*/1", "*", "*", "*", "*"}, {limit_mail_mgr, timer_check, []}},

        %% 扭蛋周一0点20清理抽奖记录
        {{20, 0, "*", "*", 1}, {mod_capsule_egg, weekly_clear, []}},

        %% 每周一0点重置扭蛋领奖次数
        {{0, 0, "*", "*", 1}, {mod_capsule_egg, week_refresh, []}},

        %% 每日3点10分经验放置清理
        {{10, 3, "*", "*", "*"}, {exp_dun_drop_mgr, daily_drop_role_check, []}},

        %% 每天1点20分社团协助记录清理
        {{20, 1, "*", "*", "*"}, {mod_guild_support, timer_clean, []}},

        %% 15分钟刷一次 材料计算
        {{"*/15", "*", "*", "*", "*"}, {mod_unite_buff_mgr, timer_check, []}},

        %% 每分钟计算
        {{"*/1", "*", "*", "*", "*"}, {mod_unite_buff_mgr, min_check, []}},

        %% 每月1号凌晨2点30分结算高级藏宝图次数奖励
        {{30, 2, 1, "*", "*"}, {mod_treasure_map_advance, monthly_reward, []}},

        %% 社团竞速，开服第一天每小时刷新一次战力榜
        {{0, "0-23/1", "*", "*", "*"}, {mod_guild_race_rank, hour_check, []}},

        %% 宠物战力榜4点上传一次到跨服
        {{0, 4, "*", "*", "*"}, {mod_pet_support_mgr, midnight_upload_pet_info, []}}
    ].
