%%-----------------------------------------------------------------------------
%%
%% the crontab running in erlang
%%
%% 1. 基本格式(Format):
%% {{m,  h, dom, mon, dow}, {M, F, A}}.
%% {{分, 时, 日, 月, 周}, {M, F, A}}
%% M:F(A)在新建的一个独立临时进程运行
%%
%% 2. 字段取值范围(Field Range):
%%  field               allowed values
%%  -----               --------------
%%  m(minute)           0-59
%%  h(hour)             0-23
%%  dom(day of month)   1-31
%%  mon(month)          1-12
%%  dow(day of week)    1-7
%%
%% 3. 用例(Usage):
%% {{"*/1", "*", "*", "*", "*"}, {io, format, ["crontab~n"]}}.
%% 每分钟输出crontab
%%
%% {{"*", "*/1", "*", "*", "*"}, {io, format, ["crontab~n"]}}.
%% 每小时输出crontab
%%
%% {{0, 3, "*", "*", "*"}, {io, format, ["crontab~n"]}}.
%% 每天凌晨3:0输出crontab
%%
%% {{0, 3, "*", "*", 1}, {io, format, ["crontab~n"]}}.
%% 每周星期一凌晨3:0输出crontab
%%
%% {{0, 21, "1-3,26-28", "*", "*"}, {io, format, ["crontab~n"]}}.
%% 每月1-3号,26-28号的晚上21:00输出crontab
%%
%% {{0, "0-23/2", "*", "*", "*"}, {io, format, ["crontab~n"]}}.
%% 每天0-23小时每隔2小时,即(0,2,4,6,8 ...22)输出crontab
%%-----------------------------------------------------------------------------

%% 跨服中心crontab配置
-module(data_crontab_cls).
-export([get_config/0]).

get_config() ->
	[
        %% 每分钟输出crontab
        %%{{"*/1", "*", "*", "*", "*"}, {io, format, ["crontab~n"]}}

        %% 定制活动开启
        {{"*/1", "*", "*", "*", "*"}, {mod_custom_act_kf, apply_cast, [timer_check, []]}},

        %% 活动开启
        {{"*/1", "*", "*", "*", "*"}, {mod_activitycalen, timer_check, []}},

        %% 1vn
        {{"*", "*/1", "*", "*", "*"}, {mod_kf_1vn_mgr, check_open, []}},

        %% 活跃度管理
        {{"*/30", "*", "*", "*", "*"}, {svr_vitality_center_mgr, timer_notice_upload, []}},

        %% 巅峰竞技：每赛季结束次日0:2结算
        {{2, 0, "*", "*", "*"}, {mod_top_pk_rank_center, daily_update, []}},

        %% 每5分钟更新榜单
        {{"*/5", "*", "*", "*", "*"}, {god_dun_rank_mgr, timer_update, []}},

        %% 每10分钟更新榜单
        % {{"*/10", "*", "*", "*", "*"}, {cross_rank_mgr, timer_update, []}}

		%% 4点02分更新跨服宠物战力榜
		{{2, 4, "*", "*", "*"}, {mod_pet_support_center, midnight_tidy_up_rank, []}}
    ].

