%%%-------------------------------------------------------------------
%%% @author liushl
%%% @doc
%%%     物品额外数据统一处理
%%% @end
%%%-------------------------------------------------------------------
-module(goods_extra_data).
-author("liushl").
-include("def_goods.hrl").
-include("goods.hrl").
-include("equipment.hrl").
%% API
-export([
    extra_data_init/1,          %% 获得物品时，初始化额外数据
    extra_data_save/1,          %% 获得物品时，保存额外数据
    extra_data_load/1,          %% 初始化物品时，初始化额外数据
    extra_data_get/3,           %% 获得额外数据
    extra_data_update/2,        %% 更新额外数据
    extra_data_del/2,           %% 删除额外数据
    extra_data_pack/1           %% 打包额外数据
]).


%%---------------------------------------------------------------------
%% 获得物品时，初始化额外数据
%%---------------------------------------------------------------------
extra_data_init(#goods{type = ?GOODS_TYPE_EQUIP} = GoodsInfo) ->
    NewGoodsInfo = equipment_api:when_init_equipment(GoodsInfo),
    NewGoodsInfo;
extra_data_init(#goods{type = ?GOODS_TYPE_GALAXY} = GoodsInfo) ->
    NewGoodsInfo = lib_role_galaxy:init_galaxy_goods(GoodsInfo),
    NewGoodsInfo;
extra_data_init(GoodsInfo) ->
    GoodsInfo.

%%---------------------------------------------------------------------
%% 获得物品时，保存额外数据
%%---------------------------------------------------------------------
extra_data_save(_GoodsInfo) ->
    ok.

%%---------------------------------------------------------------------
%% 加载物品时，初始化额外数据
%%---------------------------------------------------------------------
extra_data_load(GoodsInfo) ->
    GoodsInfo.

%%---------------------------------------------------------------------
%% 获得额外数据
%%---------------------------------------------------------------------
extra_data_get(#goods{extra_data = ExtraData}, K, Default) ->
    case lists:keyfind(K, 1, ExtraData) of
        {K, V} -> V;
        false ->
            Default
    end;
extra_data_get(_, _, Default) -> Default.
%%---------------------------------------------------------------------
%% 更新额外数据
%%---------------------------------------------------------------------
extra_data_update(GoodsInfo, []) ->
    GoodsInfo;
extra_data_update(GoodsInfo, [{K, V}|T]) ->
    ExtraData = lists:keystore(K, 1, GoodsInfo#goods.extra_data, {K, V}),
    extra_data_update(GoodsInfo#goods{extra_data = ExtraData}, T).
%%---------------------------------------------------------------------
%% 删除额外数据
%%---------------------------------------------------------------------
extra_data_del(GoodsInfo, Key) ->
    ExtraData = lists:keydelete(Key, 1, GoodsInfo#goods.extra_data),
    GoodsInfo#goods{extra_data = ExtraData}.
%%---------------------------------------------------------------------
%% 打包额外数据
%%---------------------------------------------------------------------
extra_data_pack(GoodsInfo) ->
    [
        {K, util:term_to_bitstring(V)}
        || {K, V} <- GoodsInfo#goods.extra_data
    ].
