%%% -------------------------------------------------------
%%% @author huangyongxing@yeah.net
%%% @doc
%%% 简单测试list和maps的查询及遍历效率对比
%%% @create 2020-11-13
%%%
%%% erlang otp 22.3 window10 pc测试结果
%%% （早期版本maps的遍历不是BIF，要先转换lists再遍历，应该性能更差）
%%%
%%% 测试结果显示，
%%% 对于插入和查询为主的数据，应优先使用maps
%%% 对于经常需要遍历的数据，应优先使用lists
%%% 注：maps底层数据结构，元素数量小于等于32为flatmap，多于32为hashmap
%%%
%%% > test_maplist:test_query(1000000).
%%% query_list [total: 11828(11937)ms avg: 11.828(11.937)us]
%%% query_maps [total: 31(31)ms avg: 0.031(0.031)us]
%%% ========================================================================
%%%           query_list =  11828.00ms [  100.00%]  11937.00ms [  100.00%]
%%%           query_maps =     31.00ms [    0.26%]     31.00ms [    0.26%]
%%% ========================================================================
%%% ok
%%%
%%% > test_maplist:test_for(100000).
%%% for_list [total: 13734(13781)ms avg: 137.340(137.810)us]
%%% for_maps [total: 38250(48156)ms avg: 382.500(481.560)us]
%%% ========================================================================
%%%             for_list =  13734.00ms [  100.00%]  13781.00ms [  100.00%]
%%%             for_maps =  38250.00ms [  278.51%]  48156.00ms [  349.44%]
%%% ========================================================================
%%% ok
%%%
%%% @end
%%% -------------------------------------------------------
-module(test_maplist).
-export([
        test_query/1
        ,test_for/1
    ]).

test_query(N) ->
    L = gen_list(10000),
    M = maps:from_list(L),
    ptester:run(N, [
            {query_list, fun(_) -> lists:keyfind(5000, 1, L) end},
            {query_maps, fun(_) -> maps:get(5000, M, false) end}
        ]).

test_for(N) ->
    L = gen_list(10000),
    M = maps:from_list(L),
    ptester:run(N, [
            {for_list, fun(_) -> lists:foldl(fun(_, A) -> A end, 1, L) end},
            {for_maps, fun(_) -> maps:fold(fun(_K, _V, A) -> A end, 1, M) end}
        ]).

gen_list(N) ->
    gen_list(N, []).
gen_list(C, A) when C > 0 ->
    gen_list(C - 1, [{C, C} | A]);
gen_list(_C, A) ->
    A.

